@@ -19,8 +19,11 @@
 #define _LOGINDEFS_H_ 1
 
 extern int getlogindefs_bool (const char *name, int dflt);
-long getlogindefs_num (const char *name, long dflt);
-unsigned long getlogindefs_unum (const char *name, unsigned long dflt);
-const char *getlogindefs_str (const char *name, const char *dflt);
+extern long getlogindefs_num (const char *name, long dflt);
+extern unsigned long getlogindefs_unum (const char *name, unsigned long dflt);
+extern const char *getlogindefs_str (const char *name, const char *dflt);
+
+/* Free all data allocated by getlogindefs_* calls before.  */
+extern void free_getlogindefs_data (void);
 
 #endif /* _LOGINDEFS_H_ */
