@@ -391,7 +391,7 @@ do_getpwnam (const char *user, const char *use_service)
       } while (status == NSS_STATUS_TRYAGAIN && errno == ERANGE);
     }
 
-  if (data->service != S_LOCAL)
+  if (data->service != S_LOCAL && data->service != S_NONE)
     dlclose (nss_handle);
 
   if (status == NSS_STATUS_SUCCESS)
