@@ -43,6 +43,7 @@
 #include "logindefs.h"
 #include "read-files.h"
 #include "utf8conv.h"
+#include "logging.h"
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -264,6 +265,7 @@ main (int argc, char *argv[])
   textdomain(PACKAGE);
 
   openlog (program, LOG_PID, LOG_AUTHPRIV);
+  open_sec_log();
 
   /* Before going any further, raise the ulimit and ignore
      signals.  */
@@ -334,6 +336,7 @@ main (int argc, char *argv[])
 	case 'P':
 	  if (uid != 0)
 	    {
+	      sec_log (program, MSG_PATH_ARG_DENIED, uid);
 	      fprintf (stderr,
 		       _("Only root is allowed to specify another path\n"));
 	      return E_NOPERM;
@@ -458,6 +461,8 @@ main (int argc, char *argv[])
 #endif
 		  ))
 	{
+	  sec_log (program, MSG_PERMISSION_DENIED,
+		   pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
 	  fprintf (stderr,
 		   _("Only an administrator is allowed to change aging information.\n"));
 	  free_user_t (pw_data);
@@ -504,18 +509,28 @@ main (int argc, char *argv[])
   else
 #endif /* USE_LDAP */
     if (do_authentication (program, caller_name, pw_data) != 0)
-      return E_NOPERM;
+      {
+	sec_log (program, MSG_PERMISSION_DENIED,
+		 pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
+	free_user_t (pw_data);
+	return E_NOPERM;
+      }
   /* We don't need to extra ask for a password with "-l" and if the
      password is stored in the local file.  */
     else if (!l_flag && pw_data->service != S_LOCAL)
       if (get_old_clear_password (pw_data) != 0)
-	return E_FAILURE;
+	{
+	  free_user_t (pw_data);
+	  return E_FAILURE;
+	}
 
   if (l_flag)
     {
       if (uid != 0 && pw_data->service != S_LDAP &&
 	  strcmp (caller_name, pw_data->pw.pw_name) != 0)
 	{
+	  sec_log (program, MSG_PERMISSION_DENIED,
+		   pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
 	  fprintf (stderr,
 		   _("You can only list your own aging information.\n"));
 	  return E_NOPERM;
@@ -531,7 +546,11 @@ main (int argc, char *argv[])
         }
 
       if (pw_data->use_shadow)
-	print_shadow_info (pw_data);
+	{
+	  sec_log (program, MSG_SHADOW_DATA_PRINTED,
+		   pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
+	  print_shadow_info (pw_data);
+	}
       else
 	fprintf (stdout, _("No aging information available for %s.\n"),
 		 utf8_to_locale (pw_data->pw.pw_name));
@@ -706,6 +725,47 @@ main (int argc, char *argv[])
 	printf (_("Aging information changed.\n"));
     }
 
+#if defined HAVE_LIBLAUS
+  if (pw_data->sp.sp_min != pw_data->spn.sp_min)
+    LAUS_LOG(NO_TAG, "chage: password minimum age changed - user=%s, uid=%u, min=%li, oldmin=%li, by=%u",
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_min, pw_data->sp.sp_min, uid);
+  if (pw_data->sp.sp_max != pw_data->spn.sp_max)
+    LAUS_LOG(NO_TAG, "chage: password maximum age changed - user=%s, uid=%u, max=%li, oldmax=%li, by=%u",
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_max, pw_data->sp.sp_max, uid);
+  if (pw_data->sp.sp_warn != pw_data->spn.sp_warn)
+    LAUS_LOG(NO_TAG, "chage: password warning days changed - user=%s, uid=%u, warn=%li, oldwarn=%li, by=%u",
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_warn, pw_data->sp.sp_warn, uid);
+  if (pw_data->sp.sp_inact != pw_data->spn.sp_inact)
+    LAUS_LOG(NO_TAG, "chage: password inactive days changed - user=%s, uid=%u, inactive=%li, oldinactive=%li, by=%u",
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_inact, pw_data->sp.sp_inact, uid);
+
+  if (pw_data->sp.sp_lstchg != pw_data->spn.sp_lstchg)
+    {
+      char *new_lstchg, *old_lstchg;
+
+      new_lstchg = date2str (pw_data->spn.sp_lstchg * DAY);
+      old_lstchg = date2str (pw_data->sp.sp_lstchg * DAY);
+      LAUS_LOG(NO_TAG, "chage: password last change date changed - user=%s, uid=%u, change=%s, oldchange=%s, by=%u",
+	       pw_data->pw.pw_name, pw_data->pw.pw_uid, new_lstchg, old_lstchg, uid);
+
+      free(new_lstchg);
+      free(old_lstchg);
+    }
+  if (pw_data->sp.sp_expire != pw_data->spn.sp_expire)
+    {
+      char *new_exp, *old_exp;
+
+      new_exp = date2str (pw_data->spn.sp_expire * DAY);
+      old_exp = date2str (pw_data->sp.sp_expire * DAY);
+      LAUS_LOG(NO_TAG, "chage: password expiration date changed - user=%s, uid=%u, expire=%s, oldexpire=%s, by=%u",
+	       pw_data->pw.pw_name, pw_data->pw.pw_uid, new_exp, old_exp, uid);
+
+      free(new_exp);
+      free(old_exp);
+    }
+#endif
+
+
   free_user_t (pw_data);
 
   return 0;
