@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Thorsten Kukuk
+/* Copyright (C) 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -40,8 +40,10 @@
 #include <sys/types.h>
 
 #include "public.h"
+#include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
+#include "parse_crypt_arg.h"
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -51,13 +53,6 @@
 #define _(String) gettext (String)
 #endif
 
-#ifndef RANDOM_DEVICE
-#define RANDOM_DEVICE "/dev/urandom"
-#endif
-
-enum crypt_t {DES, MD5, BLOWFISH};
-typedef enum crypt_t crypt_t;
-
 static void
 print_usage (FILE *stream, const char *program)
 {
@@ -88,102 +83,12 @@ print_help (const char *program)
   fputs (_("Valid services for -r are: files, nis, nisplus, ldap\n"), stdout);
 }
 
-#if defined(HAVE_CRYPT_GENSALT_RN)
-static int
-read_loop (int fd, char *buffer, int count)
-{
-  int offset, block;
-
-  offset = 0;
-  while (count > 0)
-    {
-      block = read(fd, &buffer[offset], count);
-
-      if (block < 0)
-        {
-          if (errno == EINTR)
-            continue;
-          return block;
-        }
-      if (!block)
-        return offset;
-
-      offset += block;
-      count -= block;
-    }
-
-  return offset;
-}
-#endif
-
-static char *
-#if defined(HAVE_CRYPT_GENSALT_RN)
-make_crypt_salt (const char *crypt_prefix, int crypt_rounds)
-#else
-make_crypt_salt (const char *crypt_prefix __attribute__ ((unused)),
-                 int crypt_rounds __attribute__ ((unused)))
-#endif
-{
-#if defined(HAVE_CRYPT_GENSALT_RN)
-#define CRYPT_GENSALT_OUTPUT_SIZE (7 + 22 + 1)
-  int fd;
-  char entropy[16];
-  char *retval;
-  char output[CRYPT_GENSALT_OUTPUT_SIZE];
-
-  fd = open (RANDOM_DEVICE, O_RDONLY);
-  if (fd < 0)
-    {
-      fprintf (stderr, _("Can't open %s for reading: %s\n"),
-	       RANDOM_DEVICE, strerror (errno));
-      return NULL;
-    }
-
-  if (read_loop (fd, entropy, sizeof(entropy)) != sizeof(entropy))
-    {
-      close (fd);
-      fprintf (stderr, _("Unable to obtain entropy from %s\n"),
-	       RANDOM_DEVICE);
-      return NULL;
-    }
-
-  close (fd);
-
-  retval = crypt_gensalt_rn (crypt_prefix, crypt_rounds, entropy,
-                             sizeof (entropy), output, sizeof(output));
-
-  memset (entropy, 0, sizeof (entropy));
-
-  if (!retval)
-    {
-      fprintf (stderr,
-	       _("Unable to generate a salt, check your crypt settings.\n"));
-      return NULL;
-    }
-
-  return strdup (retval);
-#else
-#define ascii_to_bin(c) ((c)>='a'?(c-59):(c)>='A'?((c)-53):(c)-'.')
-#define bin_to_ascii(c) ((c)>=38?((c)-38+'a'):(c)>=12?((c)-12+'A'):(c)+'.')
-
-  time_t tm;
-  char salt[3];
-
-  time (&tm);
-  salt[0] = bin_to_ascii(tm & 0x3f);
-  salt[1] = bin_to_ascii((tm >> 6) & 0x3f);
-  salt[2] = '\0';
-
-  return strdup (salt);
-#endif
-}
-
 int
 main (int argc, char *argv[])
 {
   FILE *input = NULL;
   const char *program = "chpasswd";
-  crypt_t use_crypt = DES;
+  crypt_t use_crypt;
   int encrypted = 0;
   char *buf = NULL;
   size_t buflen = 0;
@@ -200,6 +105,8 @@ main (int argc, char *argv[])
 
   openlog (program, LOG_PID, LOG_AUTHPRIV);
 
+  use_crypt = parse_crypt_arg (getlogindefs_str ("CRYPT", "des"));
+
   while (1)
     {
       int c;
@@ -233,19 +140,7 @@ main (int argc, char *argv[])
 	  files_etc_dir = strdup (optarg);
           break;
 	case 'c':
-	  if (strcasecmp (optarg, "des") == 0)
-	    use_crypt = DES;
-	  else if (strcasecmp (optarg, "md5") == 0)
-	    use_crypt = MD5;
-	  else if (strcasecmp (optarg, "blowfish") == 0 ||
-		   strcasecmp (optarg, "bf") == 0)
-	    {
-#if defined(HAVE_CRYPT_GENSALT_RN)
-	      use_crypt = BLOWFISH;
-#else
-	      fprintf (stderr, _("No support for blowfish compiled in"));
-#endif
-	    }
+	  use_crypt = parse_crypt_arg (optarg);
 	  break;
 	case 'e':
 	  ++encrypted;
@@ -280,7 +175,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2003");
+          print_version (program, "2004");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -442,6 +337,9 @@ main (int argc, char *argv[])
 	      break;
 	    case BLOWFISH:
 #if defined(HAVE_CRYPT_GENSALT_RN)
+	      /* blowfish has a limit of 72 characters */
+	      if (use_crypt == BLOWFISH && strlen (cp) > 72)
+		cp[72] = '\0';
 	      salt = make_crypt_salt ("$2a$", 0 /* XXX crypt_rounds */);
 	      if (salt != NULL)
 		pw_data->newpassword = strdup (crypt_r (cp, salt, &output));
@@ -457,11 +355,6 @@ main (int argc, char *argv[])
 	    default:
 	      abort();
 	    }
-
-	  /* blowfish has a limit of 72 characters */
-	  if (use_crypt == BLOWFISH && strlen (cp) > 72)
-	    cp[72] = '\0';
-
 	}
       time (&now);
       pw_data->spn.sp_lstchg = (long int)now / (24L*3600L);
