@@ -532,7 +532,10 @@ main (int argc, char *argv[])
       else
 #endif /* USE_LDAP */
 	if (do_authentication (program, caller_name, pw_data) != 0)
-	  return 1;
+	  {
+	    free_user_t (pw_data);
+	    return 1;
+	  }
       if (pw_data->service != S_LOCAL && get_old_clear_password (pw_data) != 0)
 	return 1;
     }
