@@ -47,6 +47,7 @@
 #endif
 
 #include "group.h"
+#include "logging.h"
 #include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
@@ -97,6 +98,7 @@ main (int argc, char **argv)
   textdomain(PACKAGE);
 
   openlog (program, LOG_PID, LOG_AUTHPRIV);
+  open_sec_log ();
 
   /* Before going any further, raise the ulimit and ignore
      signals.  */
@@ -232,6 +234,9 @@ main (int argc, char **argv)
           else
             fprintf (stderr, _("%s: Unknown group `%s'.\n"), program,
 		     group);
+
+	  sec_log (program, MSG_UNKNOWN_GROUP, group, getuid ());
+
           return E_NOPERM;
         }
     }
@@ -241,6 +246,8 @@ main (int argc, char **argv)
       (gr_data->service == S_LDAP && binddn == NULL) ||
       (gr_data->service != S_LOCAL && gr_data->service != S_LDAP))
     {
+      sec_log (program, MSG_PERMISSION_DENIED,
+	       gr_data->gr.gr_name, gr_data->gr.gr_gid, getuid ());
       fprintf (stderr, _("%s: Permission denied.\n"), program);
       free_group_t (gr_data);
       return E_NOPERM;
@@ -255,6 +262,7 @@ main (int argc, char **argv)
       char *p1, *p2;
       int try;
 
+      sec_log (program, MSG_CHANGING_GROUP_PASSWORD, group, getuid ());
       fprintf (stdout, _("Changing the password for group %s.\n"), group);
 
       for (try = 0; try < MAX_PASSWD_TRIES; try++)
@@ -264,6 +272,9 @@ main (int argc, char **argv)
 	    {
 	      fputs ("\n", stderr);
 	    abort_change:
+	      sec_log (program, MSG_PASSWORD_CHANGE_ABORTED,
+		       group, getuid ());
+
 	      fprintf (stderr, _("Password change aborted.\n"));
 	      return E_FAILURE;
 	    }
@@ -283,6 +294,7 @@ main (int argc, char **argv)
       if (try == MAX_PASSWD_TRIES)
 	{
 	  fprintf (stderr, _("%s: Try again later.\n"), program);
+	  sec_log (program, MSG_MAX_GROUP_PASSWD_TRIES, group, getuid ());
 	  goto abort_change;
         }
       else
@@ -353,11 +365,21 @@ main (int argc, char **argv)
     {
       fprintf (stderr, _("%s: Error changing password.\n"),
                program);
+      sec_log (program, MSG_ERROR_CHANGE_GROUP_PASSWORD, group, getuid ());
       free_group_t (gr_data);
       return E_FAILURE;
     }
 
-  printf (_("Password changed.\n"));
+  if (strcmp(gr_data->newpassword,"") == 0)
+    {
+      printf (_("Password removed.\n"));
+      sec_log (program, MSG_GROUP_PASSWORD_REMOVED, group, getuid ());
+    }
+  else
+    {
+      printf (_("Password changed.\n"));
+      sec_log (program, MSG_GROUP_PASSWORD_CHANGED, group, getuid ());
+    }
 
   nscd_flush_cache ("group");
 
