@@ -31,7 +31,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
-#include <syslog.h>
 #include <libgen.h>
 #include <locale.h>
 #include <libintl.h>
@@ -47,6 +46,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "logging.h"
 #include "utf8conv.h"
 #include "logindefs.h"
 #include "read-files.h"
@@ -259,6 +259,7 @@ find_free_gid (int is_system_account, int have_extrapath)
 	{
 	  fprintf (stderr, _("%s: Can't get unique gid in range %u - %u.\n"),
 		   program, gid_min, gid_max);
+	  sec_log (program, MSG_NO_FREE_GID, gid_min, gid_max);
 	  exit (E_FAILURE);
 	}
     }
@@ -410,6 +411,8 @@ main (int argc, char **argv)
       struct passwd resultbuf;
       struct passwd *pw;
 
+      open_sec_log ();
+
       /* Determine our own user name for PAM authentication.  */
       while (getpwuid_r (getuid (), &resultbuf, buffer, buflen, &pw) != 0
 	     && errno == ERANGE)
@@ -428,7 +431,11 @@ main (int argc, char **argv)
 
       if (do_authentication ("shadow" /* XXX program */,
 			     pw->pw_name, NULL) != 0)
-	return E_NOPERM;
+	{
+	  sec_log (program, MSG_PERMISSION_DENIED, pw->pw_name,
+		   pw->pw_uid, getuid() );
+	  return E_NOPERM;
+	}
     }
 
   /* After this, we can start creating the new group.  */
@@ -449,6 +456,7 @@ main (int argc, char **argv)
 	    {
 	      fprintf (stderr, _("%s: GID %u is not unique.\n"),
 		       program, new_gid);
+	      sec_log (program, MSG_GID_NOT_UNIQUE, new_gid, getuid ());
 	      return E_GID_IN_USE;
 	    }
 	}
@@ -459,6 +467,7 @@ main (int argc, char **argv)
     {
       fprintf (stderr, _("%s: Invalid group name `%s'.\n"),
 	       program, utf8_to_locale (new_group));
+      sec_log (program, MSG_GROUP_NAME_INVALID, new_group, getuid())
       return E_BAD_ARG;
     }
 
@@ -468,6 +477,7 @@ main (int argc, char **argv)
       lock_database () != 0)
     {
       fputs (_("Cannot lock group file: already locked.\n"), stderr);
+      sec_log (program, MSG_GROUP_FILE_ALREADY_LOCKED);
       return E_PWDBUSY;
     }
   else if (getgrnam (new_group) != NULL ||
@@ -475,10 +485,12 @@ main (int argc, char **argv)
     {				/* Group does already exists.  */
       fprintf (stderr, _("%s: Group `%s' already exists.\n"),
 	       program, utf8_to_locale (new_group));
+      sec_log (program, MSG_GROUP_ALREADY_EXISTS, new_group, getuid ());
       return E_NAME_IN_USE;
     }
   else
     {
+      int i;
       group_t gr_data;
       int retval = E_SUCCESS;
 
@@ -504,9 +516,6 @@ main (int argc, char **argv)
 
       gr_data.gr.gr_gid = know_gid ? new_gid : find_free_gid (system_account,
 							      have_extrapath);
-      syslog (LOG_INFO,
-	      "new group: name=%s, gid=%u",
-	      gr_data.gr.gr_name, (unsigned int) gr_data.gr.gr_gid);
 
 #ifdef USE_LDAP
       if (gr_data.service == S_LDAP)
@@ -542,7 +551,16 @@ main (int argc, char **argv)
 #endif
 
       if (write_group_data (&gr_data, 1) != 0)
-	return E_FAILURE;
+	{
+	  sec_log (program, MSG_ERROR_ADDING_NEW_GROUP, gr_data.gr.gr_name,
+		   (unsigned int) gr_data.gr.gr_gid, getuid());
+	  return E_FAILURE;
+	}
+      else
+	{
+	  sec_log (program, MSG_NEW_GROUP_ADDED, gr_data.gr.gr_name,
+		   (unsigned int) gr_data.gr.gr_gid, getuid());
+	}
 
       /* flush NSCD cache.  */
       nscd_flush_cache ("group");
@@ -550,15 +568,16 @@ main (int argc, char **argv)
       if (use_service == NULL || strcmp (use_service, "files") == 0)
 	ulckpwdf ();
 
-#if 0 /* XXX */
-      i = call_script ("GROUPADD_CMD", gr_data.gr.gr_name, gr_data.gr.gr_gid);
+      /* XXX sec_log calling of call_script */
+      i = call_script ("GROUPADD_CMD", gr_data.gr.gr_name,
+		       gr_data.gr.gr_gid, getuid (), NULL);
       if (i != 0)
 	{
 	  fprintf (stderr, _("%s: GROUPADD_CMD fails with exit code %d.\n"),
 		   program, i);
 	  retval = E_FAILURE;
 	}
-#endif
+
       return retval;
     }
 
