@@ -77,7 +77,7 @@ print_help (const char *program)
   fputs (_("Valid services for --service are: files, ldap\n"), stdout);
 }
 
-static const char *program = "userdel";
+static const char *program = "groupdel";
 
 static struct option long_options[] = {
 #ifdef USE_LDAP
