@@ -47,6 +47,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "utf8conv.h"
 #include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
@@ -63,8 +64,7 @@ print_help (const char *program)
   print_usage (stdout, program);
   fprintf (stdout, _("%s - modify a group entry\n\n"), program);
 
-  fputs (_
-	 ("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
+  fputs (_("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
 	 stdout);
   fputs (_("  -g gid         Change the groupid to the given number\n"),
 	 stdout);
@@ -72,11 +72,12 @@ print_help (const char *program)
 	 stdout);
   fputs (_("  -n name        Change group name.\n"), stdout);
   fputs (_("  -o             Allow duplicate (non-unique) UID\n"), stdout);
-  fputs (_
-	 ("  -P path        Search passwd, shadow and group file in \"path\"\n"),
+  fputs (_("  -P path        Search passwd, shadow and group file in \"path\"\n"),
 	 stdout);
   fputs (_("  -p password    Encrypted password as returned by crypt(3)\n"),
 	 stdout);
+  fputs (_("  -A user        Add the user to the group entry\n"), stdout);
+  fputs (_("  -R user        Remove the user from the group entry\n"), stdout);
   fputs (_("  -r service     Use nameservice 'service'\n"), stdout);
   fputs (_("      --help     Give this help list\n"), stdout);
   fputs (_("      --usage    Give a short usage message\n"), stdout);
@@ -95,13 +96,15 @@ static struct option long_options[] = {
   {"non-unique", no_argument, NULL, 'o'},
   {"path", required_argument, NULL, 'P'},
   {"password", required_argument, NULL, 'p'},
+  {"add-user", required_argument, NULL, 'A'},
+  {"remove-user", required_argument, NULL, 'R'},
   {"service", required_argument, NULL, 'r'},
   {"version", no_argument, NULL, 'v'},
   {"usage", no_argument, NULL, '\254'},
   {"help", no_argument, NULL, '\255'},
   {NULL, 0, NULL, '\0'}
 };
-static const char *short_options = "D:g:n:oP:p:r:u:v";
+static const char *short_options = "A:D:g:n:oP:p:R:r:v";
 
 static struct group *
 files_getgrnam (const char *name)
@@ -153,6 +156,70 @@ files_getgrgid (gid_t gid)
     return NULL;
 }
 
+/* XXX move to libpwdutils.  */
+static char **
+remove_gr_mem (const char *name, char **gr_mem)
+{
+  char **groups;
+  unsigned int i, j;
+
+  for (i = 0; gr_mem[i]; i++) ;
+  ++i;                          /* for trailing NULL pointer */
+
+  groups = malloc (i * sizeof (char *));
+  for (i = 0, j = 0; gr_mem[i]; i++)
+    {
+      if (strcmp (name, gr_mem[i]) != 0)
+        {
+          groups[j] = strdup (gr_mem[i]);
+          ++j;
+        }
+    }
+
+  groups[j] = NULL;
+
+  return groups;
+}
+
+/* XXX move to libpwdutils.  */
+static char **
+add_gr_mem (const char *name, char **gr_mem)
+{
+  char **groups;
+  unsigned int i;
+  int already_added = 0;
+
+  i = 0;
+  while (gr_mem[i])
+    {
+      if (strcmp (gr_mem[i], name) == 0)
+        already_added = 1;
+      ++i;
+    }
+  ++i;                          /* for trailing NULL pointer */
+
+  if (!already_added)
+    ++i;
+
+  groups = malloc (i * sizeof (char *));
+  i = 0;
+  while (gr_mem[i])
+    {
+      groups[i] = strdup (gr_mem[i]);
+      ++i;
+    }
+
+  if (!already_added)
+    {
+      groups[i] = strdup (name);
+      ++i;
+    }
+
+  groups[i] = NULL;
+
+  return groups;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -166,6 +233,8 @@ main (int argc, char **argv)
   char *new_password = NULL;
   gid_t new_gid = 0;
   char *know_gid = NULL;
+  char *remove_user = NULL;
+  char *add_user = NULL;
   int non_unique = 0;
   int have_extrapath = 0;
   int retval = E_SUCCESS;
@@ -205,7 +274,7 @@ main (int argc, char **argv)
 	  know_gid = optarg;
 	  break;
 	case 'n':
-	  new_name = optarg;
+	  new_name = locale_to_utf8 (optarg);
 	  break;
 	case 'o':
 	  non_unique = 1;
@@ -230,6 +299,14 @@ main (int argc, char **argv)
 	    }
 	  new_password = optarg;
 	  break;
+	case 'R':
+	  if (remove_user != NULL)
+	    {
+	      print_usage (stderr, program);
+	      return E_BAD_ARG;
+	    }
+	  remove_user = locale_to_utf8 (optarg);
+	  break;
 	case 'r':
 	  if (use_service != NULL)
 	    {
@@ -250,6 +327,14 @@ main (int argc, char **argv)
 	      return E_BAD_ARG;
 	    }
 	  break;
+	case 'A':
+	  if (add_user != NULL)
+	    {
+	      print_usage (stderr, program);
+	      return E_BAD_ARG;
+	    }
+	  add_user = locale_to_utf8 (optarg);
+	  break;
 	case '\255':
 	  print_help (program);
 	  return 0;
@@ -307,24 +392,16 @@ main (int argc, char **argv)
 	return E_NOPERM;
     }
 
-  modify_group = argv[0];
+  modify_group = locale_to_utf8 (argv[0]);
   /* Check, if the account we should modify exist.  */
   gr_data = find_group_data (modify_group, 0, use_service);
   if (gr_data == NULL || gr_data->service == S_NONE)
-    {				/* User does not exist.  */
-      fprintf (stderr, _("%s: user `%s' does not exist.\n"),
-	       program, modify_group);
+    {				/* Group does not exist.  */
+      fprintf (stderr, _("%s: group `%s' does not exist.\n"),
+	       program, utf8_to_locale (modify_group));
       return E_NOTFOUND;
     }
 
-  /* Check if user is logged in.  */
-  if (is_logged_in (modify_group))
-    {
-      fprintf (stderr, _("%s: account `%s' is currently in use.\n"),
-	       program, modify_group);
-      return E_USER_BUSY;
-    }
-
   /* After this, we can start modifying the existing account.  */
   if (know_gid != NULL && !non_unique)
     {
@@ -344,7 +421,7 @@ main (int argc, char **argv)
       if (check_name (new_name) != 0)
 	{
 	  fprintf (stderr, _("%s: Invalid group name `%s'.\n"),
-		   program, new_name);
+		   program, utf8_to_locale (new_name));
 	  return E_BAD_ARG;
 	}
       else
@@ -353,7 +430,7 @@ main (int argc, char **argv)
 	      (have_extrapath && files_getgrnam (new_name) != NULL))
 	    {
 	      fprintf (stderr, _("%s: Group `%s' already exists.\n"),
-		       program, new_name);
+		       program, utf8_to_locale (new_name));
 	      return E_NAME_IN_USE;
 	    }
 	}
@@ -367,10 +444,8 @@ main (int argc, char **argv)
       fputs (_("Cannot lock password file: already locked.\n"), stderr);
       return E_PWDBUSY;
     }
-  else if (new_name || new_password || know_gid)
+  else
     {
-      /* Only change group file if there are really changes.  */
-
       gr_data->todo = DO_MODIFY;
       if (new_name)
 	gr_data->new_name = strdup (new_name);
@@ -382,6 +457,12 @@ main (int argc, char **argv)
 	  gr_data->new_gid = new_gid;
 	}
 
+      if (remove_user)
+	gr_data->new_gr_mem = remove_gr_mem (remove_user, gr_data->gr.gr_mem);
+
+      if (add_user)
+	gr_data->new_gr_mem = add_gr_mem (add_user, gr_data->gr.gr_mem);
+
 #ifdef USE_LDAP
       if (gr_data->service == S_LDAP)
 	{
