@@ -476,7 +476,7 @@ loop_over_shadow_file (int quiet)
   if (input == NULL)
     {
       /* fprintf (stderr, _("Can't open %s: %m\n"), inputname); */
-      return 0;
+      return E_NO_FILE;
     }
 
   if (!quiet)
@@ -1003,5 +1003,21 @@ main (int argc, char *argv[])
   if (sort)
     return sort_passwd_file ();
   else
-    return loop_over_passwd_file (quiet) + loop_over_shadow_file (quiet);
+    {
+      int rp, rs;
+
+      rp = loop_over_passwd_file (quiet);
+      rs = loop_over_shadow_file (quiet);
+
+      if (rs == 0 || rs == E_NO_FILE)
+	return rp;
+
+      if (rp == 0 || rp == E_NO_FILE)
+	return rs;
+
+      if (rp > rs)
+	return rp;
+      else
+	return rs;
+    }
 }
