@@ -82,8 +82,60 @@ send_string (SSL *ssl, u_int32_t ret, const char *str)
 struct slpcb {
   char *srvurl;
   SLPError err;
+  struct slpcb *next;
+  char *hostp;
+  char *portp;
+  char *descr;
 };
 
+static void
+free_slpcb (struct slpcb *cb)
+{
+  struct slpcb *tcb;
+
+  free (cb->srvurl);
+  cb = cb->next;
+
+  while (cb)
+    {
+      tcb = cb;
+      cb = cb->next;
+      free (tcb->srvurl);
+      if (tcb->descr)
+	free (tcb->descr);
+      free (tcb);
+    }
+}
+
+static void
+parse_slpcb (struct slpcb *cb)
+{
+#define NEEDLE "://"
+  size_t needle_length = strlen (NEEDLE);
+
+  cb->hostp = strstr (cb->srvurl, NEEDLE);
+
+  if (cb->hostp == NULL || strlen (cb->hostp) < needle_length + 1)
+    return;
+
+  cb->hostp += needle_length;
+
+  cb->portp = strchr (cb->hostp, ':');
+  if (cb->portp)
+    {
+      char *cp;
+
+      cb->portp[0] = '\0';
+      cb->portp += 1;
+      cp = cb->portp;
+      while (isdigit (*cp))
+	cp++;
+
+      if (*cp != '\0')
+	*cp = '\0';
+    }
+}
+
 static SLPBoolean
 MySLPSrvURLCallback (SLPHandle hslp __attribute__((unused)),
 		     const char *srvurl,
@@ -94,27 +146,66 @@ MySLPSrvURLCallback (SLPHandle hslp __attribute__((unused)),
 
   if (errcode == SLP_OK)
     {
+      if (cb->srvurl != NULL)
+        {
+          struct slpcb *cbt = malloc (sizeof (struct slpcb));
+          if (cbt == NULL)
+            return SLP_FALSE;
+
+	  cbt->srvurl = cb->srvurl;
+	  cbt->hostp = cb->hostp;
+	  cbt->portp = cb->portp;
+	  cbt->err = cb->err;
+	  cbt->next = cb->next;
+	  cb->next = cbt;
+	  cb->descr = NULL;
+        }
       cb->srvurl = strdup (srvurl);
+      parse_slpcb (cb);
       cb->err = SLP_OK;
-      /* We don't won't more currently.  */
-      return SLP_FALSE;
+      return SLP_TRUE;
     }
-  else
+  else if (errcode != SLP_LAST_CALL)
     cb->err = errcode;
 
   return SLP_FALSE; /* We don't wan't to be called again.  */
 }
 
-static int
-query_slp (HANDLE char **hostp, char **portp)
+static SLPBoolean
+MySLPAttrCallback (SLPHandle hslp __attribute__((unused)),
+		   const char *attrlist, SLPError errcode, void *cookie )
 {
-  struct slpcb callbackres = {NULL, 0};
+  char **descr = (char **)cookie;
+
+  if (errcode == SLP_OK)
+    {
+      char *cp = strstr (attrlist, "(description=");
+
+      if (cp == NULL)
+	return SLP_FALSE;
+
+      *descr = strdup (cp+13);
+
+      cp = strchr (*descr, ')');
+      if (cp != NULL)
+	*cp = '\0';
+    }
+
+  return SLP_FALSE;
+}
+
+static int
+query_slp (HANDLE char **hostp, char **portp, char **descrp)
+{
+  struct slpcb *cb, callbackres = {NULL, 0, NULL, NULL, NULL, NULL};
   SLPError err;
   SLPHandle hslp;
 
-
   *hostp = NULL;
   *portp = NULL;
+  *descrp = NULL;
+
+  PRINTF (STD_HANDLE, _("Searching a server...\n"));
 
   err = SLPOpen ("en", SLP_FALSE, &hslp);
   if (err != SLP_OK)
@@ -134,40 +225,79 @@ query_slp (HANDLE char **hostp, char **portp)
       return -1;
     }
 
+  cb = &callbackres;
+
+  while (cb != NULL)
+    {
+      char *buf;
+
+      asprintf (&buf, "service:rpasswdd://%s:%s/", cb->hostp,
+		cb->portp?:"774");
+      err = SLPFindAttrs (hslp, buf, "",
+			  "", /* use configured scopes */
+			  MySLPAttrCallback, &(cb->descr));
+      free (buf);
+      if (err != SLP_OK)
+	{
+	  PRINTF (STD_HANDLE, _("Error while searching for SLP description.\n"));
+	  return -1;
+	}
+      cb = cb->next;
+    }
+
   /* Now that we're done using slp, close the slp handle */
   SLPClose (hslp);
 
-  if (callbackres.srvurl != NULL)
+  cb = &callbackres;
+#if defined(SELECT_SRVURL)
+  if (cb->next != NULL) /* Only if we have more than one entry.  */
     {
-#define NEEDLE "://"
-      size_t needle_length = strlen (NEEDLE);
-      *hostp = strstr (callbackres.srvurl, NEEDLE);
+      /* Ask the user which one to use.  */
+      struct slpcb *tcb = &callbackres;
+      int choice = 0;     /* index of the user's choice.  */
+      char response[20];  /* string to hold the user's response.  */
+      int i = 0;
 
-      if (*hostp == NULL || strlen (*hostp) < needle_length + 1)
+      printf (_("\nPlease select a server:\n"));
+      while (tcb != NULL)
 	{
-	  free (callbackres.srvurl);
-	  return -1;
+	  ++i;
+	  printf ("[%2d] %s", i, tcb->hostp);
+	  if (tcb->portp)
+	    printf (_(" (port %s)"), tcb->portp);
+	  if (tcb->descr)
+	    printf (" - %s", tcb->descr);
+	  fputs ("\n", stdout);
+	  tcb = tcb->next;
 	}
 
-      *hostp += needle_length;
-
-      *portp = strchr (*hostp, ':');
-      if (*portp)
+      while ((choice < 1) || (choice > i))
 	{
-	  char *cp;
-
-	  *portp[0] = '\0';
-	  *portp += 1;
-	  cp = *portp;
-	  while (isdigit (*cp))
-	    {
-	      cp++;
-	    }
-	  if (*cp != '\0')
-	    *cp = '\0';
+	  printf ("Enter number of choice: ");
+	  fflush (stdin);
+	  fgets (response, sizeof (response), stdin);
+	  fflush (stdin);
+	  choice = strtol (response, NULL, 10);
 	}
+      printf ("\n");
+      for (i = 0; i < (choice - 1); i++)
+	cb = cb->next;
+    }
+#endif
+
+  if (cb->hostp != NULL)
+    {
+      *hostp = strdup (cb->hostp);
+      if (cb->portp)
+	*portp = strdup (cb->portp);
+      if (cb->descr)
+	*descrp = strdup (cb->descr);
+
+      free_slpcb (&callbackres);
       return 0;
     }
+
+  free_slpcb (&callbackres);
   return -1;
 }
 #endif
