@@ -64,6 +64,7 @@
 #define PRINTF fprintf
 #define ERR_HANDLE stderr
 #define STD_HANDLE stdout
+#define SELECT_SRVURL 1
 
 #include "rpasswd-client.c"
 
@@ -418,12 +419,22 @@ main (int argc, char **argv)
 #ifdef USE_SLP
   if (hostp == NULL && portp == NULL && use_slp == 1)
     {
-      query_slp (&hostp, &portp);
+      char *descr;
+
+      query_slp (&hostp, &portp, &descr);
       if (hostp != NULL && portp != NULL)
-	printf (_("SLP: Found Server on %s, port %s.\n"),
+	printf (_("SLP: Found Server on %s, port %s"),
 		hostp, portp);
       else if (hostp != NULL)
-        printf (_("SLP: Found Server on %s.\n"), hostp);
+        printf (_("SLP: Found Server on %s"), hostp);
+
+      if (descr)
+	{
+	  printf (" (%s)\n", descr);
+	  free (descr);
+	}
+      else
+	fputs ("\n", stdout);
     }
 #endif
 
