@@ -74,7 +74,8 @@ extern int setresuid(uid_t ruid, uid_t euid, uid_t suid);
 /* XXX This variable should not be global.  */
 static SSL_CTX *ctx;
 /* Socket for incoming connections.  */
-static int sock4 = -1, sock6 = -1;
+static struct pollfd pollfd_conn[10];
+static int pollfd_cnt = 0;
 #ifdef USE_SLP
 /* register/deregister at SLP server.  */
 static int use_slp = 0;
@@ -97,7 +98,7 @@ static void
 print_usage (FILE *stream, const char *program)
 {
 #ifdef USE_SLP
-  fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n       [--slp [--slp-timeout timeout]]\n"),
+  fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n       [--slp [--slp-timeout timeout] [--slp-descr description]\n"),
 	   program);
 #else
   fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n"),
@@ -122,6 +123,8 @@ print_help (const char *program)
 #ifdef USE_SLP
   fputs (_("  --slp          Register at local SLP server\n"), stdout);
   fputs (_("  --slp-timeout  Set timeout for re-registration\n"), stdout);
+  fputs (_("  --slp-descr    Set a description shown to SLP clients\n"),
+	 stdout);
 #endif
   fputs (_("  --help         Give this help list\n"), stdout);
   fputs (_("  --usage        Give a short usage message\n"), stdout);
@@ -202,8 +205,8 @@ server_init (int port, int ipv4, int ipv6)
       int on = 1;
 
       /* Create the socket.  */
-      sock6 = socket (AF_INET6, SOCK_STREAM, 0);
-      if (sock6 < 0)
+      pollfd_conn[pollfd_cnt].fd = socket (AF_INET6, SOCK_STREAM, 0);
+      if (pollfd_conn[pollfd_cnt].fd < 0)
 	{
 	  dbg_log (_("cannot open socket: %s"), strerror (errno));
 	  exit (1);
@@ -214,8 +217,8 @@ server_init (int port, int ipv4, int ipv6)
 	 for IPv4. If the IPV6_V6ONLY option failed, we assume IPv6
 	 will also handle IPv4 addresses.  */
 
-      if (setsockopt (sock6, IPPROTO_IPV6, IPV6_V6ONLY,
-			  (char *)&on, sizeof (on)) <0)
+      if (setsockopt (pollfd_conn[pollfd_cnt].fd, IPPROTO_IPV6, IPV6_V6ONLY,
+		      (char *)&on, sizeof (on)) <0)
 	{
 	  if (errno != ENOPROTOOPT)
 	    dbg_log ("setsockopt (IPPROTO_IPV6, IPV6_V6ONLY): %m");
@@ -228,19 +231,21 @@ server_init (int port, int ipv4, int ipv6)
       sock_addr.sin6_addr = in6addr_any;
       sock_addr.sin6_port = port;
 
-      if (bind (sock6, (struct sockaddr *) &sock_addr, sizeof (sock_addr)) < 0)
+      if (bind (pollfd_conn[pollfd_cnt].fd, (struct sockaddr *) &sock_addr,
+		sizeof (sock_addr)) < 0)
 	{
 	  dbg_log ("bind: %s", strerror (errno));
 	  exit (1);
 	}
 
       /* Set the socket up to accept connections.  */
-      if (listen (sock6, SOMAXCONN) < 0)
+      if (listen (pollfd_conn[pollfd_cnt].fd, SOMAXCONN) < 0)
 	{
 	  dbg_log (_("cannot enable socket to accept connections: %s"),
 		   strerror (errno));
 	  exit (1);
 	}
+      ++pollfd_cnt;
     }
 
   if (ipv4 && (!ipv6 || have_usagi))
@@ -248,8 +253,8 @@ server_init (int port, int ipv4, int ipv6)
       struct sockaddr_in sock_addr;
 
       /* Create the socket.  */
-      sock4 = socket (AF_INET, SOCK_STREAM, 0);
-      if (sock4 < 0)
+      pollfd_conn[pollfd_cnt].fd = socket (AF_INET, SOCK_STREAM, 0);
+      if (pollfd_conn[pollfd_cnt].fd < 0)
 	{
 	  dbg_log (_("cannot open socket: %s"), strerror (errno));
 	  exit (1);
@@ -260,19 +265,21 @@ server_init (int port, int ipv4, int ipv6)
       sock_addr.sin_addr.s_addr = INADDR_ANY;
       sock_addr.sin_port = port;
 
-      if (bind (sock4, (struct sockaddr *) &sock_addr, sizeof (sock_addr)) < 0)
+      if (bind (pollfd_conn[pollfd_cnt].fd, (struct sockaddr *) &sock_addr,
+		sizeof (sock_addr)) < 0)
 	{
 	  dbg_log ("bind: %s", strerror (errno));
 	  exit (1);
 	}
 
       /* Set the socket up to accept connections.  */
-      if (listen (sock4, SOMAXCONN) < 0)
+      if (listen (pollfd_conn[pollfd_cnt].fd, SOMAXCONN) < 0)
 	{
 	  dbg_log (_("cannot enable socket to accept connections: %s"),
 		   strerror (errno));
 	  exit (1);
 	}
+      ++pollfd_cnt;
     }
 }
 
@@ -280,10 +287,11 @@ server_init (int port, int ipv4, int ipv6)
 static void
 close_sockets (void)
 {
-  if (sock4 >= 0)
-    close (sock4);
-  if (sock6 >= 0)
-    close (sock6);
+  int i;
+
+  for (i = 0; i < pollfd_cnt; i++)
+    if (pollfd_conn[i].fd >= 0)
+      close (pollfd_conn[i].fd);
 }
 
 static int
@@ -707,22 +715,16 @@ cannot handle request version %d; current version is %d"),
 static void
 server_run (void)
 {
-  struct pollfd conn[2];
-  int pollfd_cnt = 0;
+  int i;
 
-  if (sock4 >= 0)
-    conn[pollfd_cnt++].fd = sock4;
-  if (sock6 >= 0)
-    conn[pollfd_cnt++].fd = sock6;
-
-  conn[0].events = POLLRDNORM;
-  conn[1].events = POLLRDNORM;
+  for (i = 0; i < pollfd_cnt; i++)
+    pollfd_conn[i].events = POLLRDNORM;
 
   while (1)
     {
-      int i, nr;
+      int nr;
 
-      nr = poll (conn, pollfd_cnt, -1);
+      nr = poll (pollfd_conn, pollfd_cnt, -1);
 
       if (nr < 0)
         {
@@ -736,7 +738,7 @@ server_run (void)
       /* We have a new incoming connection. Look at which socket.  */
       for (i = 0; i < pollfd_cnt; i++)
 	{
-	  if (conn[i].revents & (POLLRDNORM|POLLERR|POLLHUP|POLLNVAL))
+	  if (pollfd_conn[i].revents & (POLLRDNORM|POLLERR|POLLHUP|POLLNVAL))
 	    {
 	      /* Accept the connection.  */
 	      int fd, ssl_err;
@@ -745,7 +747,7 @@ server_run (void)
 	      SSL *ssl;
 	      char *locale, *username;
 
-	      fd = accept (conn[i].fd, NULL, NULL);
+	      fd = accept (pollfd_conn[i].fd, NULL, NULL);
 	      if (fd < 0)
 		{
 		  dbg_log (_("while accepting connection: %s"),
@@ -1019,7 +1021,10 @@ main (int argc, char **argv)
   char *certificate = "/etc/rpasswdd.pem";
   char *privatekey = "/etc/rpasswdd.pem";
   int port = -1;
+#ifdef USE_SLP
   int slp_timeout = 3600;
+  char *slp_descr = NULL;
+#endif
   SSL_METHOD *meth;
 
   /* Set locale via LC_ALL.  */
@@ -1040,6 +1045,7 @@ main (int argc, char **argv)
 	  {"ipv6",     no_argument,       NULL, '6'},
 	  {"certificate", required_argument, NULL, 'c'},
 	  {"privatekey",  required_argument, NULL, 'k'},
+	  {"slp-descr",   required_argument, NULL, '\250'},
 	  {"slp-timeout", required_argument, NULL, '\251'},
 	  {"slp",      no_argument,       NULL, '\252'},
           {"version",  no_argument,       NULL, '\255'},
@@ -1074,6 +1080,9 @@ main (int argc, char **argv)
 	  port = htons (atol (optarg));
 	  break;
 #ifdef USE_SLP
+	case '\250':
+	  slp_descr = optarg;
+	  break;
 	case '\251':
 	  slp_timeout = atol (optarg);
 	  break;
@@ -1207,7 +1216,7 @@ main (int argc, char **argv)
 #ifdef USE_SLP
   /* Register at local SLP server.  */
   if (use_slp)
-    register_slp (ntohs(port), slp_timeout);
+    register_slp (ntohs(port), slp_timeout, slp_descr);
 #endif
 
   /* Handle incoming requests.  */
