@@ -371,17 +371,17 @@ no integer value >= -1 in %s.\n"), program, &cp[7], configfile);
 
 	  continue;
 	}
-      else if (strncasecmp (cp, "SKEL=", 6) == 0)
+      else if (strncasecmp (cp, "SKEL=", 5) == 0)
 	{
-	  if (access (&cp[6], F_OK) != 0)
+	  if (access (&cp[5], F_OK) != 0)
 	    {
 	      ++errors;
 	      fprintf (stderr,
 		       _("%s: Skel directory \"%s\" in %s does not exist.\n"),
-		       program, &cp[6], configfile);
+		       program, &cp[5], configfile);
 	    }
 	  else
-	    data->skel = strdup (&cp[6]);
+	    data->skel = strdup (&cp[5]);
 	  continue;
 	}
       else if (strncasecmp (cp, "GROUPS=", 7) == 0)
@@ -449,7 +449,7 @@ print_defaults (const struct default_t *data)
   if (data->expire == -1)
     fputs ("EXPIRE=\n", stdout);
   else
-    printf ("EXPIRE=%s\n", date2str (data->expire));
+    printf ("EXPIRE=%s\n", date2str (data->expire * SCALE));
   printf ("SHELL=%s\n", data->shell);
   printf ("SKEL=%s\n", data->skel);
   printf ("GROUPS=");
@@ -584,7 +584,7 @@ write_defaults (const char *configfile, struct default_t *data)
 	  if (data->expire < 0)
 	    fputs ("EXPIRE=\n", new_fp);
 	  else
-	    fprintf (new_fp, "EXPIRE=%s\n", date2str (data->expire));
+	    fprintf (new_fp, "EXPIRE=%s\n", date2str (data->expire * SCALE));
 	  expire_written = 1;
 	  continue;
 	}
@@ -594,7 +594,7 @@ write_defaults (const char *configfile, struct default_t *data)
 	  shell_written = 1;
 	  continue;
 	}
-      else if (strncasecmp (cp, "SKEL=", 6) == 0)
+      else if (strncasecmp (cp, "SKEL=", 5) == 0)
 	{
 	  fprintf (new_fp, "SKEL=%s\n", data->skel);
 	  skel_written = 1;
@@ -633,7 +633,7 @@ write_defaults (const char *configfile, struct default_t *data)
       if (data->expire < 0)
 	fputs ("EXPIRE=\n", new_fp);
       else
-	fprintf (new_fp, "EXPIRE=%s\n", date2str (data->expire));
+	fprintf (new_fp, "EXPIRE=%s\n", date2str (data->expire * SCALE));
     }
   if (!shell_written)
     fprintf (new_fp, "SHELL=%s\n", data->shell);
