@@ -70,8 +70,7 @@ print_help (const char *program)
 	 ("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
  stdout);
   fputs (_("  -d homedir     Home directory for the new user\n"), stdout);
-  fputs (_
-	 ("  -e expire      Date on which the new account will be disabled\n"),
+  fputs (_("  -e expire      Date on which the new account will be disabled\n"),
 	 stdout);
   fputs (_("  -f inactive    Days after a password expires until account is \
 disabled\n"), stdout);
@@ -83,8 +82,11 @@ disabled\n"), stdout);
   fputs (_("  -l login       Change login name.\n"), stdout);
   fputs (_("  -m             Move home directory to the new path\n"), stdout);
   fputs (_("  -o             Allow duplicate (non-unique) UID\n"), stdout);
-  fputs (_
-	 ("  -P path        Search passwd, shadow and group file in \"path\"\n"),
+  fputs (_("  -A group,...   List of groups the user should be added to\n"),
+	 stdout);
+  fputs (_("  -R group,...   List of groups the user should be removed from\n"),
+	 stdout);
+  fputs (_("  -P path        Search passwd, shadow and group file in \"path\"\n"),
 	 stdout);
   fputs (_("  -p password    Encrypted password as returned by crypt(3)\n"),
 	 stdout);
@@ -114,6 +116,8 @@ static struct option long_options[] = {
   {"expire", required_argument, NULL, 'e'},
   {"inactive", required_argument, NULL, 'f'},
   {"groups", required_argument, NULL, 'G'},
+  {"add-to-groups", required_argument, NULL, 'A'},
+  {"remove-from-groups", required_argument, NULL, 'R'},
   {"gid", required_argument, NULL, 'g'},
   {"login", required_argument, NULL, 'l'},
   {"lock", required_argument, NULL, 'L'},
@@ -130,7 +134,7 @@ static struct option long_options[] = {
   {"help", no_argument, NULL, '\255'},
   {NULL, 0, NULL, '\0'}
 };
-static const char *short_options = "c:D:d:e:f:G:g:l:LmoP:p:r:s:u:Uv";
+static const char *short_options = "A:c:D:d:e:f:G:g:l:LmoP:p:R:r:s:u:Uv";
 
 /* This function converts a comma seperated list of groups
    (a group can be a groupname or a group id) into a table
@@ -724,8 +728,12 @@ main (int argc, char **argv)
   char *new_login = NULL;
   char *new_password = NULL;
   char *new_groups_arg = NULL;
+  char *add_groups_arg = NULL;
+  char *remove_groups_arg = NULL;
   char **new_groups = NULL;
   unsigned int new_groupscnt = 0;
+  char **del_groups = NULL;
+  unsigned int del_groupscnt = 0;
   uid_t new_uid = 0;
   char *know_uid = NULL;
   int non_unique = 0;
@@ -825,10 +833,35 @@ main (int argc, char **argv)
 	  }
 	  break;
 	case 'G':
+	  if (add_groups_arg || remove_groups_arg)
+	    {
+	      print_usage (stderr, program);
+	      return E_BAD_ARG;
+	    }
 	  /* Only save the arguments for later checking. We can find a
 	     -P <path> option later.  */
 	  new_groups_arg = locale_to_utf8 (optarg);
 	  break;
+	case 'A':
+	  if (new_groups_arg)
+	    {
+	      print_usage (stderr, program);
+	      return E_BAD_ARG;
+	    }
+	  /* Only save the arguments for later checking. We can find a
+	     -P <path> option later.  */
+	  add_groups_arg = locale_to_utf8 (optarg);
+	  break;
+	case 'R':
+	  if (new_groups_arg)
+	    {
+	      print_usage (stderr, program);
+	      return E_BAD_ARG;
+	    }
+	  /* Only save the arguments for later checking. We can find a
+	     -P <path> option later.  */
+	  remove_groups_arg = locale_to_utf8 (optarg);
+	  break;
 	case 'g':
 	  new_group = locale_to_utf8 (optarg);
 	  break;
@@ -1065,6 +1098,121 @@ main (int argc, char **argv)
 	}
     }
 
+  /* Create list of groups, to which we should add this account. -A option.  */
+  /* XXX merge with new_groups_arg above.  */
+  if (add_groups_arg)
+    {
+      char *arg = add_groups_arg;
+      unsigned int errors = 0, i, j;
+
+      j = 1;
+      for (i = 0; i < strlen (arg); i++)
+	if (arg[i] == ',')
+	  ++j;
+
+      new_groups = malloc (sizeof (char *) * j);
+      new_groupscnt = 0;
+
+      do
+	{
+	  char *cp = strchr (arg, ',');
+	  if (cp)
+	    *cp++ = '\0';
+
+	  if (arg && *arg)
+	    {
+	      gid_t gid;
+
+	      if (convert_grpopt_to_name (arg,
+					  &new_groups[new_groupscnt],
+					  &gid, use_service) != 0)
+		++errors;
+	      else
+		{
+		  /* If this group is the primary group, print error
+		     message and ignore it. Don't exit with an error,
+		     this would break too many existing scripts.  */
+		  if ((new_group &&
+		       strcmp (new_group, new_groups[new_groupscnt]) == 0) ||
+		      (gid == pw_data->pw.pw_gid))
+		    {
+		      fprintf (stderr, _("%s: `%s' is primary group name.\n"),
+			       program,
+			       utf8_to_locale (new_groups[new_groupscnt]));
+		      /* return E_BAD_ARG; */
+		    }
+		  else
+		    {
+		      new_groupscnt++;
+		      if (new_groupscnt > j)
+			abort ();
+		    }
+		}
+	    }
+	  arg = cp;
+	}
+      while (arg);
+
+      if (errors)
+	{
+	  /* This is more a guess than something else.  */
+	  if (files_etc_dir)
+	    return E_NOTFOUND;
+	  else
+	    return E_BAD_ARG;
+	}
+    }
+
+  /* Create list of groups, to which we should add this account. -R option.  */
+  /* XXX merge with new_groups_arg above.  */
+  if (remove_groups_arg)
+    {
+      char *arg = remove_groups_arg;
+      unsigned int errors = 0, i, j;
+
+      j = 1;
+      for (i = 0; i < strlen (arg); i++)
+	if (arg[i] == ',')
+	  ++j;
+
+      del_groups = malloc (sizeof (char *) * j);
+      del_groupscnt = 0;
+
+      do
+	{
+	  char *cp = strchr (arg, ',');
+	  if (cp)
+	    *cp++ = '\0';
+
+	  if (arg && *arg)
+	    {
+	      gid_t gid;
+
+	      if (convert_grpopt_to_name (arg,
+					  &del_groups[del_groupscnt],
+					  &gid, use_service) != 0)
+		++errors;
+	      else
+		{
+		  del_groupscnt++;
+		  if (del_groupscnt > j)
+		    abort ();
+		}
+	    }
+	  arg = cp;
+	}
+      while (arg);
+
+      if (errors)
+	{
+	  /* This is more a guess than something else.  */
+	  if (files_etc_dir)
+	    return E_NOTFOUND;
+	  else
+	    return E_BAD_ARG;
+	}
+    }
+
   /* Check if user is logged in.  */
   if ((new_login || know_uid || new_home) &&
       is_logged_in (modify_account))
@@ -1211,7 +1359,7 @@ main (int argc, char **argv)
 
   /* Change the login name in group entries, too.
      But only, if we don't need to remove them all later.  */
-  if (new_login && new_groupscnt == 0)
+  if (new_login && (new_groups_arg == NULL || new_groupscnt == 0))
     {
       if (rename_in_secondary_groups (pw_data, have_extrapath) != 0)
 	retval = E_FAILURE;
@@ -1220,22 +1368,24 @@ main (int argc, char **argv)
       nscd_flush_cache ("group");
     }
 
-  /* Now we need to change the group file (-G option).  */
-  if (new_groups_arg)
+  /* Now we need to change the group file (-G/-A option).  */
+  if (new_groups_arg || add_groups_arg)
     {
       unsigned int i;
       int ret;
 
-      /* Remove user from all groups. */
-      if (new_login)
-	ret = remove_from_secondary_groups (pw_data, have_extrapath,
-					    new_login);
-      else
-	ret = remove_from_secondary_groups (pw_data, have_extrapath,
-					    pw_data->pw.pw_name);
-
-      if (ret != 0)
-	retval = ret;
+      if (new_groups_arg)
+	{
+	  /* Remove user from all groups. */
+	  if (new_login)
+	    ret = remove_from_secondary_groups (pw_data, have_extrapath,
+						new_login);
+	  else
+	    ret = remove_from_secondary_groups (pw_data, have_extrapath,
+						pw_data->pw.pw_name);
+	  if (ret != 0)
+	    retval = ret;
+	}
 
       for (i = 0; i < new_groupscnt; i++)
 	{
@@ -1325,10 +1475,99 @@ main (int argc, char **argv)
 	}
     }
 
-#if 0
-  /* Now we need to change the group file (-A option).  */
-  XXX
+  /* Now we need to change the group file (-R option).  */
+  /* XXX merge with above.  */
+  if (remove_groups_arg)
+    {
+      unsigned int i;
+
+      for (i = 0; i < del_groupscnt; i++)
+	{
+	  group_t *gr_data = find_group_data (del_groups[i], 0, use_service);
+
+
+	  if (gr_data == NULL || gr_data->service == S_NONE)
+	    {
+	      if (use_service == NULL)
+		fprintf (stderr,
+			 _("%s: ERROR: cannot find group `%s' anymore!.\n"),
+			 program, utf8_to_locale (del_groups[i]));
+	      else
+		fprintf (stderr,
+			 _
+			 ("%s: Cannot find group `%s' in service `%s', ignored.\n"),
+			 program, utf8_to_locale (del_groups[i]), use_service);
+	      retval = E_NOTFOUND;
+	    }
+	  else
+	    {
+	      gr_data->todo = DO_MODIFY;
+#ifdef USE_LDAP
+	      if (gr_data->service == S_LDAP)
+		{
+		  if (binddn == NULL)
+		    {
+		      binddn = get_caller_dn ();
+		      if (binddn == NULL)
+			{
+			  fprintf (stderr,
+				   _
+				   ("%s: Cannot remove user from groups stored in LDAP database without DN.\n"),
+				   program);
+			}
+		    }
+
+		  if (binddn == NULL)
+		    {
+		      fprintf (stderr,
+			       _("%s: User not removed from LDAP group `%s'.\n"),
+			       program,
+			       utf8_to_locale (gr_data->gr.gr_name));
+		      free_group_t (gr_data);
+		      retval = E_GRP_UPDATE;
+		      continue;
+		    }
+
+		  gr_data->binddn = strdup (binddn);
+
+		  if (pw_data->oldclearpwd == NULL)
+		    {
+		      char *cp = get_ldap_password (binddn);
+
+		      if (cp)
+			pw_data->oldclearpwd = strdup (cp);
+		      else
+			{
+			  fprintf (stderr,
+				   _("%s: User not removed from LDAP group `%s'.\n"),
+				   program,
+				   utf8_to_locale (gr_data->gr.gr_name));
+			  free_group_t (gr_data);
+			  retval = E_GRP_UPDATE;
+			  continue;
+			}
+		    }
+
+		  if (pw_data->oldclearpwd)
+		    gr_data->oldclearpwd = strdup (pw_data->oldclearpwd);
+		}
 #endif
+	      gr_data->new_gr_mem = remove_gr_mem (modify_account,
+						   gr_data->gr.gr_mem);
+	      if (write_group_data (gr_data, 1) != 0)
+		{
+		  fprintf (stderr,
+			   _("%s: User not removed from LDAP group `%s'.\n"),
+			   program,
+			   utf8_to_locale (gr_data->gr.gr_name));
+		  retval = E_GRP_UPDATE;
+		}
+	      /* Make sure, written group changes will be active now.  */
+	      nscd_flush_cache ("group");
+	    }
+	  free_group_t (gr_data);
+	}
+    }
 
   if (move_homedir)
     if (move_home_directory (old_home, new_home) != 0)
